package sk.kasv.schmidt.javafx.worldx_gui.Controller;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.stage.Stage;
import sk.kasv.schmidt.javafx.worldx_gui.API.ApiCaller;
import sk.kasv.schmidt.javafx.worldx_gui.Database.DB;
import sk.kasv.schmidt.javafx.worldx_gui.Main.WorldXApplication;
import sk.kasv.schmidt.javafx.worldx_gui.Util.utility;

import java.io.IOException;
import java.sql.SQLException;
import java.util.List;

public class WorldXController {

    @FXML
    public ChoiceBox<String> cities;

    @FXML
    public Label cityLabel;
    public Label tempOutput;
    public Label population;
    public Label populationLabel;
    public Label temperatureLabel;
    public RadioButton celsiusButton;
    public RadioButton kelvinButton;
    public RadioButton fahrenheitButton;
    public Label languageLabel;
    public CheckBox togglePopulationButton;
    public TextField countryFinder;
    public Button aboutButton;

    @FXML
    private ChoiceBox<String> countries;

    @FXML
    protected void initialize() {
        try {
            setupTemperatureButtons();
            setupCountryChoiceBox();
            setupCityChoiceBox();
            setupCountryFinder();
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    private void setupTemperatureButtons() {
        ToggleGroup group = new ToggleGroup();
        celsiusButton.setToggleGroup(group);
        fahrenheitButton.setToggleGroup(group);
        kelvinButton.setToggleGroup(group);

        celsiusButton.setSelected(true);
        celsiusButton.setDisable(true);
        fahrenheitButton.setSelected(false);
        fahrenheitButton.setDisable(true);
        kelvinButton.setSelected(false);
        kelvinButton.setDisable(true);

        celsiusButton.setOnAction(e -> setTemperatureToCelsius());
        fahrenheitButton.setOnAction(e -> setTemperatureToFahrenheit());
        kelvinButton.setOnAction(e -> setTemperatureToKelvin());

        togglePopulationButton.setDisable(false);
        togglePopulationButton.setVisible(true);
        togglePopulationButton.setOnAction(e -> population.setVisible(!population.isVisible()));
    }

    private void setTemperatureToCelsius() {
        tempOutput.setText(utility.roundToTwoDecimals(ApiCaller.getTemperature(cities.getValue(),
                DB.get3LetterCode(countries.getValue()))) + "°C");
    }

    private void setTemperatureToFahrenheit() {
        tempOutput.setText(utility.roundToTwoDecimals(ApiCaller.getTemperature(cities.getValue(),
                DB.get3LetterCode(countries.getValue())) * 9/5 + 32) + "°F");
    }

    private void setTemperatureToKelvin() {
        tempOutput.setText(utility.roundToTwoDecimals(ApiCaller.getTemperature(cities.getValue(),
                DB.get3LetterCode(countries.getValue())) + 273.15) + "°K");
    }

    private void setupCountryChoiceBox() throws SQLException {
        List<String> countryList = DB.getAllCountries();
        countries.getItems().addAll(" ");
        countries.getItems().addAll(countryList);
        countries.setOnAction(e -> {
            try {
                updateCities();
            } catch (SQLException ex) {
                throw new RuntimeException(ex);
            }
        });
    }

    private void updateCities() throws SQLException {
        List<String> cityList = DB.getCitiesByCountry(countries.getValue());
        cities.getItems().clear();
        cities.getItems().addAll(cityList);
        cities.setValue(null);
        population.setVisible(togglePopulationButton.isSelected());
    }

    private void setupCityChoiceBox() {
        cities.setOnAction(e -> updateCityInfo());
    }

    private void updateCityInfo() {
        if (cities.getValue() == null) {
            hideCityInfo();
        } else {
            showCityInfo();
        }
    }

    private void hideCityInfo() {
        tempOutput.setVisible(false);
        celsiusButton.setDisable(true);
        fahrenheitButton.setDisable(true);
        kelvinButton.setDisable(true);
        languageLabel.setVisible(false);

        if (!togglePopulationButton.isSelected()) {
            population.setVisible(false);
        }
    }

    private void showCityInfo() {
        tempOutput.setVisible(true);
        temperatureLabel.setVisible(true);
        languageLabel.setVisible(true);
        languageLabel.setText(DB.getLanguage(DB.get3LetterCode(countries.getValue())));

        celsiusButton.setDisable(false);
        celsiusButton.setSelected(true);
        fahrenheitButton.setDisable(false);
        fahrenheitButton.setSelected(false);
        kelvinButton.setDisable(false);
        kelvinButton.setSelected(false);

        setTemperatureToCelsius();

        population.setText(""+DB.getPopulation(cities.getValue()));

        if (togglePopulationButton.isSelected()) {
            population.setVisible(true);
        }
    }

    private void setupCountryFinder() {
        countryFinder.textProperty().addListener((observable, oldValue, newValue) -> {
            List<String> filteredCountries = DB.getFilteredCountries(newValue);
            countries.getItems().clear();
            countries.getItems().addAll(filteredCountries);
        });
    }

    public void openAboutForm(ActionEvent actionEvent) throws IOException {
        Stage stage = (Stage) ((Node) actionEvent.getSource()).getScene().getWindow();
        FXMLLoader fxmlLoader = new FXMLLoader(WorldXApplication.class.getResource("/sk/kasv/schmidt/javafx/worldx_gui/about.fxml"));
        Scene scene = new Scene(fxmlLoader.load(),640, 480);
        stage.setTitle("About WorldX Application");
        stage.setScene(scene);
        stage.show();
    }
}