package sk.kasv.schmidt.javafx.worldx_gui.Database;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import org.json.JSONObject;
import sk.kasv.schmidt.javafx.worldx_gui.Util.utility;

public class DB {
    private static final String URL = "jdbc:mysql://localhost:3306/world_x";
    private static final String USER = "root";
    private static final String PASS = "";
    private static final String SELECT_ALL_COUNTRIES = "SELECT Name FROM country";
    private static final String SELECT_3LETTER_CODE = "SELECT Code FROM country WHERE Name = ?";
    private static final String SELECT_CITIES_BY_COUNTRY =
            "SELECT city.Name FROM city JOIN country ON city.CountryCode = country.Code WHERE country.Name = ?";
    private static final String SELECT_POPULATION = "SELECT Info FROM city WHERE Name = ?";
    private static final String SELECT_LANGUAGE = "SELECT Language FROM countrylanguage WHERE IsOfficial = 'T' AND countrylanguage.CountryCode = ?";
    private static final String SELECT_FILTERED_COUNTRIES = "SELECT Name FROM country WHERE Name LIKE ?";

    private static Connection connection;

    static {
        try {
            connection = DriverManager.getConnection(URL, USER, PASS);
            System.out.println("Connected to the database!");
        } catch (SQLException e) {
            System.out.println("Connection failed!");
            e.printStackTrace();
        }
    }

    public static List<String> getAllCountries() throws SQLException {
        List<String> countries = new ArrayList<>();
        try (PreparedStatement preparedStatement = connection.prepareStatement(SELECT_ALL_COUNTRIES);
             ResultSet resultSet = preparedStatement.executeQuery()) {
            while (resultSet.next()) {
                countries.add(resultSet.getString("Name"));
            }
        }
        return countries;
    }

    public static List<String> getCitiesByCountry(String selectedCountry) {
        List<String> cities = new ArrayList<>();
        try (PreparedStatement preparedStatement = connection.prepareStatement(SELECT_CITIES_BY_COUNTRY)) {
            preparedStatement.setString(1, selectedCountry);
            try (ResultSet resultSet = preparedStatement.executeQuery()) {
                while (resultSet.next()) {
                    cities.add(resultSet.getString("city.Name")); // Use alias "city.Name"
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return cities;
    }

    public static String get3LetterCode(String countryName) {
        String countryCode3 = "";
        try (PreparedStatement preparedStatement = connection.prepareStatement(SELECT_3LETTER_CODE)) {
            preparedStatement.setString(1, countryName);
            try (ResultSet resultSet = preparedStatement.executeQuery()) {
                while (resultSet.next()) {
                    countryCode3 = resultSet.getString("Code");
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return countryCode3;
    }

    public static String getPopulation(String city) {
        int population = 0;
        try (PreparedStatement preparedStatement = connection.prepareStatement(SELECT_POPULATION)) {
            preparedStatement.setString(1, city);
            try (ResultSet resultSet = preparedStatement.executeQuery()) {
                while (resultSet.next()) {
                    String info = resultSet.getString("Info");
                    JSONObject json = new JSONObject(info);
                    population = json.getInt("Population");
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return utility.prettifyPopulation(population);
    }

    public static String getLanguage(String countryCode3) {
        String language = "";
        try (PreparedStatement preparedStatement = connection.prepareStatement(SELECT_LANGUAGE)) {
            preparedStatement.setString(1, countryCode3);
            try (ResultSet resultSet = preparedStatement.executeQuery()) {
                while (resultSet.next()) {
                    language = resultSet.getString("Language");
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return language;
    }

    public static List<String> getFilteredCountries(String newValue) {
        List<String> filteredCountries = new ArrayList<>();
        try (PreparedStatement preparedStatement = connection.prepareStatement(SELECT_FILTERED_COUNTRIES)) {
            preparedStatement.setString(1, newValue + "%");
            try (ResultSet resultSet = preparedStatement.executeQuery()) {
                while (resultSet.next()) {
                    filteredCountries.add(resultSet.getString("Name"));
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return filteredCountries;
    }
}